@@ -21,7 +21,7 @@
  * author : Devresse Adrien
  */
 
-#define _GNU_SOURCE
+
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,13 +40,17 @@
 #error "GFAL_PLUGIN_DIR_DEFAULT should be define at compile time"
 #endif
 
+
+
+
+
 /*
  * function to use in order to create a new plugin interface
  *  permit to keep the ABI compatibility
  *  must be use in ALL the plugin's "gfal_plugin_init" functions
  */
 gfal_plugin_interface* gfal_plugin_interface_new(){
-	size_t s_plugin = sizeof(gfal_plugin_interface) *2;
+    size_t s_plugin = sizeof(gfal_plugin_interface)+1;
 	gfal_plugin_interface* ret = malloc( s_plugin); // bigger allocation that needed for futur extensions
 	memset(ret, 0,s_plugin);
 	return ret;
@@ -56,6 +60,14 @@ plugin_handle gfal_get_plugin_handle(gfal_plugin_interface* cata_list){
     return cata_list->plugin_data;
 }
 
+gboolean gfal_feature_is_supported(void * ptr, GQuark scope, const char* func_name, GError** err){
+    if(ptr == NULL){
+       g_set_error(err, scope,EPROTONOSUPPORT, "[%s] Protocol not supported or path/url invalid", func_name);
+       return FALSE;
+    }
+    return TRUE;
+}
+
 //convenience function for safe calls to the plugin checkers
 gboolean gfal_plugin_checker_safe(gfal_plugin_interface* cata_list, const char* path, plugin_mode call_type, GError** terr ){
 	if(cata_list->check_plugin_url)
@@ -84,23 +96,38 @@ static int gfal_module_init(gfal_handle handle, void* dlhandle, const char* modu
 		handle->plugin_opt.plugin_list[*n] = constructor(handle, &tmp_err);
 		handle->plugin_opt.plugin_list[*n].gfal_data = dlhandle;
 		if(tmp_err){
-			g_prefix_error(&tmp_err, " Unable to load plugin %s : ", module_name);		
+            g_prefix_error(&tmp_err, "Unable to load plugin %s : ", module_name);
 			*n=0;		
 		}else{
 			*n+=1;
-			gfal_log(GFAL_VERBOSE_NORMAL, " [gfal_module_load] plugin %s loaded with success ", module_name);
+            gfal_log(GFAL_VERBOSE_NORMAL, "[gfal_module_load] plugin %s loaded with success ", module_name);
 			ret=0;
 		}
 	}
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-	return ret;	
+    G_RETURN_ERR(ret, tmp_err, err);
+}
+
+// unload each loaded plugin
+int gfal_plugins_delete(gfal_handle handle, GError** err){
+    g_return_val_err_if_fail(handle, -1, err, "[gfal_plugins_delete] Invalid value of handle");
+    const int plugin_number = handle->plugin_opt.plugin_number;
+    if(plugin_number > 0){
+            int i;
+            for(i=0; i< plugin_number; ++i){
+                gfal_plugin_interface* p = &(handle->plugin_opt.plugin_list[i]);
+                if(p->plugin_delete)
+                    p->plugin_delete(gfal_get_plugin_handle(p));
+            }
+
+        handle->plugin_opt.plugin_number =0;
+    }
+    return 0;
 }
 
 
 
 // return the proper plugin linked to this file handle
-static gfal_plugin_interface* gfal_plugin_getModuleFromHandle(gfal_handle handle, gfal_file_handle fh, GError** err){
+gfal_plugin_interface* gfal_plugin_map_file_handle(gfal_handle handle, gfal_file_handle fh, GError** err){
 	GError* tmp_err=NULL;
 	int i;
 	gfal_plugin_interface* cata_list=NULL;
@@ -185,11 +212,16 @@ plugin_pointer_handle gfal_plugins_list_handler(gfal_handle handle, GError** err
 	if(n > 0){
 		resu =g_new0(struct _plugin_pointer_handle,n+1);
 		int i;
-		gfal_plugin_interface* cata_list = handle->plugin_opt.plugin_list;
-		for(i=0; i < n; ++i, ++cata_list){
+        GList * plugin_list = g_list_first(handle->plugin_opt.sorted_plugin);
+        i =0;
+        while(plugin_list != NULL){
+            gfal_plugin_interface* cata_list = plugin_list->data;
 			resu[i].dlhandle = cata_list->gfal_data;
 			resu[i].plugin_data = cata_list->plugin_data;
+            resu[i].plugin_api = cata_list;
 			g_strlcpy(resu[i].plugin_name, cata_list->getName(), GFAL_URL_MAX_LEN);
+            i++;
+            plugin_list = g_list_next(plugin_list);
 		}	
 	}
 	
@@ -289,6 +321,41 @@ int gfal_modules_resolve(gfal_handle handle, GError** err){
 	return ret;
 }
 
+//
+// compare of the plugin function
+//
+gint gfal_plugin_compare(gconstpointer a, gconstpointer b){
+    gfal_plugin_interface* pa  = (gfal_plugin_interface*)  a;
+    gfal_plugin_interface* pb  = (gfal_plugin_interface*)  b;
+    return ( pa->priority > pb->priority)?(-1):(((pa->priority == pb->priority)?0:1));
+}
+
+
+//
+// Sort plugins by priority
+//
+int gfal_plugins_sort(gfal_handle handle, GError ** err){
+    int i;
+    for(i=0; i < handle->plugin_opt.plugin_number;++i){
+        handle->plugin_opt.sorted_plugin = g_list_append(handle->plugin_opt.sorted_plugin, &(handle->plugin_opt.plugin_list[i]));
+    }
+    handle->plugin_opt.sorted_plugin= g_list_sort(handle->plugin_opt.sorted_plugin, &gfal_plugin_compare);
+
+    if(gfal_get_verbose() & GFAL_VERBOSE_TRACE){ // print plugin order
+        GString* strbuff = g_string_new (" plugin priority order: ");
+        GList* l = handle->plugin_opt.sorted_plugin;
+
+        for(i=0; i < handle->plugin_opt.plugin_number;++i){
+            strbuff = g_string_append(strbuff, ((gfal_plugin_interface*) l->data)->getName());
+            strbuff = g_string_append(strbuff, " -> ");
+            l = g_list_next(l);
+        }
+        gfal_log(GFAL_VERBOSE_TRACE, "%s", strbuff->str);
+        g_string_free(strbuff, TRUE);
+    }
+    return 0;
+}
+
 //
 // Instance all plugins for use if it's not the case
 // return the number of plugin available
@@ -302,92 +369,37 @@ int gfal_plugins_instance(gfal_handle handle, GError** err){
 		if(tmp_err){
 			g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
 			handle->plugin_opt.plugin_number = -1;
-		}
+        }else if(handle->plugin_opt.plugin_number > 0){
+            gfal_plugins_sort(handle, &tmp_err);
+            if(tmp_err){
+                g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
+                return -1;
+            }
+        }
 		return handle->plugin_opt.plugin_number;	
 	}
 	return plugin_number;
 }
 
-//  generic plugin operation executor
-//  Check alls plugins, if a plugin is valid execute the given operation on this plugin and return result, 
-//  else return nagative value and set GError to the correct error
- int gfal_plugins_operation_executor(gfal_handle handle, gboolean (*checker)(gfal_plugin_interface*, GError**),
-										int (*executor)(gfal_plugin_interface*, GError**) , GError** err){
-	GError* tmp_err=NULL;
-	int i;
-	int ret = -1;
-	const int n_plugins = gfal_plugins_instance(handle, &tmp_err);
-	if(n_plugins > 0){
-		gfal_plugin_interface* cata_list = handle->plugin_opt.plugin_list;
-		for(i=0; i < n_plugins; ++i, ++cata_list){
-			const gboolean comp =  checker(cata_list, &tmp_err);
-			if(tmp_err)
-				break;
-				
-			if(comp){
-				ret = executor(cata_list, &tmp_err);
-				break;		
-			}	
-		}
-	}
-	if(tmp_err){	
-		g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-	}else if(ret){
-		g_set_error(err,0,EPROTONOSUPPORT, "[%s] Protocol not supported or path/url invalid", __func__);
-	}
-	return ret;
-	 
- }
 
- //  generic plugin operation executor with user data
- //  Check alls plugins, if a plugin is valid execute the given operation on this plugin and return result,
- //  else return nagative value and set GError to the correct error
-  int gfal_plugins_operation_executor_generic(gfal_handle handle,
-                                      gboolean (*checker)(gfal_plugin_interface*, gpointer userdata_check, GError**), gpointer user_data_check,
-                                     int (*executor)(gfal_plugin_interface*, gpointer user_data_exec, GError**) , gpointer user_data_exec,
-                                      GError** err){
-     GError* tmp_err=NULL;
-     int i;
-     int ret = -1;
-     const int n_plugins = gfal_plugins_instance(handle, &tmp_err);
-     if(n_plugins > 0){
-         gfal_plugin_interface* cata_list = handle->plugin_opt.plugin_list;
-         for(i=0; i < n_plugins; ++i, ++cata_list){
-             const gboolean comp =  checker(cata_list, user_data_check, &tmp_err);
-             if(tmp_err)
-                 break;
 
-             if(comp){
-                 ret = executor(cata_list, user_data_exec, &tmp_err);
-                 break;
-             }
-         }
-     }
-     if(tmp_err){
-         g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-     }else if(ret){
-         g_set_error(err,0,EPROTONOSUPPORT, "[%s] Protocol not supported or path/url invalid", __func__);
-     }
-     return ret;
 
-  }
-
-  gfal_plugin_interface* gfal_find_catalog(gfal_handle handle,
+gfal_plugin_interface* gfal_find_plugin(gfal_handle handle,
                                            const char * url,
                                            plugin_mode acc_mode, GError** err){
         GError* tmp_err=NULL;
-        int i;
         gboolean compatible = FALSE;
         const int n_plugins = gfal_plugins_instance(handle, &tmp_err);
         if(n_plugins > 0){
-          gfal_plugin_interface* cata_list = handle->plugin_opt.plugin_list;
-          for(i=0; i < n_plugins; ++i, ++cata_list){
+            GList * plugin_list = g_list_first(handle->plugin_opt.sorted_plugin);
+            while(plugin_list != NULL){
+              gfal_plugin_interface* cata_list = plugin_list->data;
               compatible =  gfal_plugin_checker_safe(cata_list, url, acc_mode , &tmp_err);
               if(tmp_err)
                   break;
               if(compatible)
                   return cata_list;
-
+              plugin_list = g_list_next(plugin_list);
           }
         }
         if(tmp_err){
@@ -406,113 +418,75 @@ int gfal_plugins_instance(gfal_handle handle, GError** err){
 //  result of the access method or -1 if error and set GError with the correct value
 //  error : EPROTONOSUPPORT means that the URL is not matched by a plugin
 int gfal_plugins_accessG(gfal_handle handle, const char* path, int mode, GError** err){
-	g_return_val_err_if_fail(handle && path, EINVAL, err, "[gfal_plugins_accessG] Invalid arguments");
-	GError* tmp_err=NULL;
-	
-	gboolean access_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_ACCESS, terr);	
-	}
-	int access_executor(gfal_plugin_interface* cata_list, GError** terr){
-		return cata_list->accessG(cata_list->plugin_data, path, mode, terr);
-	}
-	
-	int ret = gfal_plugins_operation_executor(handle, &access_checker, &access_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-	return ret;
+    g_return_val_err_if_fail(handle && path, EINVAL, err, "[gfal_plugins_accessG] Invalid arguments");
+    int res=-1;
+    GError * tmp_err=NULL;
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_ACCESS, &tmp_err);
+
+    if(p)
+        res = p->accessG(gfal_get_plugin_handle(p), path, mode, &tmp_err);
+
+    G_RETURN_ERR(res, tmp_err, err);
 }
 
 //  Execute a stat function on the appropriate plugin
 int gfal_plugin_statG(gfal_handle handle, const char* path, struct stat* st, GError** err){
 	g_return_val_err_if_fail(handle && path, EINVAL, err, "[gfal_plugin_statG] Invalid arguments");
-	GError* tmp_err=NULL;
+    int ret =-1;
+    GError* tmp_err=NULL;
 	
-	gboolean stat_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_STAT, terr);
-	}
-	int stat_executor(gfal_plugin_interface* cata_list, GError** terr){
-		return cata_list->statG(cata_list->plugin_data, path, st, terr);
-	}
-	
-	int ret = gfal_plugins_operation_executor(handle, &stat_checker, &stat_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__); 
-	return ret;	
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_STAT, &tmp_err);
+
+    if(p)
+        ret = p->statG(gfal_get_plugin_handle(p), path, st, &tmp_err);
+
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
+
+//  Execute a stat function on the appropriate plugin
+int gfal_plugin_lstatG(gfal_handle handle, const char* path, struct stat* st, GError** err){
+    g_return_val_err_if_fail(handle && path, EINVAL, err, "[gfal_plugin_statG] Invalid arguments");
+    int ret =-1;
+    GError* tmp_err=NULL;
+
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_LSTAT, &tmp_err);
+
+    if(p)
+        ret = p->lstatG(gfal_get_plugin_handle(p), path, st, &tmp_err);
+
+    G_RETURN_ERR(ret, tmp_err, err);
+}
+
+
 //  Execute a readlink function on the appropriate plugin
 ssize_t gfal_plugin_readlinkG(gfal_handle handle, const char* path, char* buff, size_t buffsiz, GError** err){
-	g_return_val_err_if_fail(handle && path, EINVAL, err, "[gfal_plugin_readlinkG] Invalid arguments");
-	GError* tmp_err=NULL;
-	ssize_t resu;
-	
-	gboolean readlink_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_READLINK, terr);
-	}
-	int readlink_executor(gfal_plugin_interface* cata_list, GError** terr){
-		return ((resu = cata_list->readlinkG(cata_list->plugin_data, path, buff, buffsiz, terr)) !=-1)?0:-1;
-	}
-	
-	gfal_plugins_operation_executor(handle, &readlink_checker, &readlink_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__); 
-	return resu;	
+    g_return_val_err_if_fail(handle && path, EINVAL, err, "[gfal_plugin_readlinkG] Invalid arguments");
+    GError* tmp_err=NULL;
+    ssize_t resu=-1;
+
+   gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_READLINK, &tmp_err);
+
+   if(p)
+       resu = p->readlinkG(gfal_get_plugin_handle(p), path, buff, buffsiz, &tmp_err);
+
+    G_RETURN_ERR(resu, tmp_err, err);
 }
 
 
 
-
-//  Execute a lstat function on the appropriate plugin 
-int gfal_plugin_lstatG(gfal_handle handle, const char* path, struct stat* st, GError** err){
-	g_return_val_err_if_fail(handle && path, EINVAL, err, "[gfal_plugin_lstatG] Invalid arguments");
-	GError* tmp_err=NULL;
-	
-	gboolean lstat_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_LSTAT, terr);
-	}
-	int lstat_executor(gfal_plugin_interface* cata_list, GError** terr){
-		return cata_list->lstatG(cata_list->plugin_data, path, st, terr);
-	}
-	
-	int ret = gfal_plugins_operation_executor(handle, &lstat_checker, &lstat_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__); 
-	return ret;	
-}
-
-// unload each loaded plugin
-int gfal_plugins_delete(gfal_handle handle, GError** err){
-	g_return_val_err_if_fail(handle, -1, err, "[gfal_plugins_delete] Invalid value of handle");
-	const int plugin_number = handle->plugin_opt.plugin_number;
-	if(plugin_number > 0){
-			int i;
-			for(i=0; i< plugin_number; ++i){
-				if(handle->plugin_opt.plugin_list[i].plugin_delete)
-					handle->plugin_opt.plugin_list[i].plugin_delete( handle->plugin_opt.plugin_list[i].plugin_data );
-			}
-		
-		handle->plugin_opt.plugin_number =0;
-	}
-	return 0;
-}
-
 //  Execute a chmod function on the appropriate plugin 
  int gfal_plugin_chmodG(gfal_handle handle, const char* path, mode_t mode, GError** err){
 	g_return_val_err_if_fail(handle && path, -1, err, "[gfal_plugin_chmodG] Invalid arguments");	
-	GError* tmp_err = NULL;		
+    GError* tmp_err = NULL;
+    int ret = -1;
+
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_CHMOD, &tmp_err);
+
+    if(p)
+        ret = p->chmodG(gfal_get_plugin_handle(p), path, mode, &tmp_err);
 	
-		
-	gboolean chmod_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_CHMOD, terr);	
-	}
-	int chmod_executor(gfal_plugin_interface* cata_list, GError** terr){
-		return cata_list->chmodG(cata_list->plugin_data, path, mode, terr);
-	}
-	
-	int ret = gfal_plugins_operation_executor(handle, &chmod_checker, &chmod_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret; 
+    G_RETURN_ERR(ret, tmp_err, err);
  }
  
 
@@ -520,76 +494,56 @@ int gfal_plugins_delete(gfal_handle handle, GError** err){
 int gfal_plugin_renameG(gfal_handle handle, const char* oldpath, const char* newpath, GError** err){
 	g_return_val_err_if_fail(oldpath && newpath, -1, err, "[gfal_plugin_renameG] invalid value in args oldpath, handle or newpath");
 	GError* tmp_err=NULL;
+    int ret = -1;
+    gfal_plugin_interface* p;
+    if( (p = gfal_find_plugin(handle, oldpath, GFAL_PLUGIN_RENAME, &tmp_err)) != NULL
+         && (p = gfal_find_plugin(handle, newpath, GFAL_PLUGIN_RENAME, &tmp_err)) != NULL){
+        ret = p->renameG(gfal_get_plugin_handle(p), oldpath, newpath, &tmp_err);
+    }
 	
-	gboolean rename_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return (gfal_plugin_checker_safe(cata_list, oldpath, GFAL_PLUGIN_RENAME, terr) && 
-			gfal_plugin_checker_safe(cata_list, newpath, GFAL_PLUGIN_RENAME, terr));	
-	}
-	int rename_executor(gfal_plugin_interface* cata_list, GError** terr){
-		return cata_list->renameG(cata_list->plugin_data, oldpath, newpath, terr);
-	}
-	
-	int ret = gfal_plugins_operation_executor(handle, &rename_checker, &rename_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret; 
-	
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 // Execute a symlink function on the appropriate plugin 
 int gfal_plugin_symlinkG(gfal_handle handle, const char* oldpath, const char* newpath, GError** err){
 	g_return_val_err_if_fail(oldpath && newpath, -1, err, "[gfal_plugin_symlinkG] invalid value in args oldpath, handle or newpath");
 	GError* tmp_err=NULL;
+    int ret = -1;
+    gfal_plugin_interface* p;
 	
-	gboolean symlink_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return (gfal_plugin_checker_safe(cata_list, oldpath, GFAL_PLUGIN_SYMLINK, terr) && 
-			gfal_plugin_checker_safe(cata_list, newpath, GFAL_PLUGIN_SYMLINK, terr));	
-	}
-	int symlink_executor(gfal_plugin_interface* cata_list, GError** terr){
-		return cata_list->symlinkG(cata_list->plugin_data, oldpath, newpath, terr);
-	}
-	
-	const int ret = gfal_plugins_operation_executor(handle, &symlink_checker, &symlink_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret; 	
-	
+    if( (p = gfal_find_plugin(handle, oldpath, GFAL_PLUGIN_SYMLINK, &tmp_err)) != NULL
+         && (p = gfal_find_plugin(handle, newpath, GFAL_PLUGIN_SYMLINK, &tmp_err)) != NULL){
+        ret = p->symlinkG(gfal_get_plugin_handle(p), oldpath, newpath, &tmp_err);
+    }
+
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 // Execute a mkdir function on the appropriate plugin 
 int gfal_plugin_mkdirp(gfal_handle handle, const char* path, mode_t mode, gboolean pflag,  GError** err){
 	g_return_val_err_if_fail(handle && path , -1, err, "[gfal_plugin_mkdirp] Invalid argumetns in path or/and handle");
 	GError* tmp_err=NULL;	
+    int ret = -1;
 
-	gboolean mkdirp_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_MKDIR, terr);	
-	}
-	int mkdirp_executor(gfal_plugin_interface* cata_list, GError** terr){
-		return cata_list->mkdirpG(cata_list->plugin_data, path, mode, pflag, terr);
-	}
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_MKDIR, &tmp_err);
 
-	int ret = gfal_plugins_operation_executor(handle, &mkdirp_checker, &mkdirp_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret; 	
+    if(p)
+        ret = p->mkdirpG(gfal_get_plugin_handle(p), path, mode, pflag, &tmp_err);
+
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 // Execute a rmdir function on the appropriate plugin 
 int gfal_plugin_rmdirG(gfal_handle handle, const char* path, GError** err){
 	g_return_val_err_if_fail(handle && path , -1, err, "[gfal_plugin_rmdirp] Invalid arguments in path or/and handle");
 	GError* tmp_err=NULL;	
+    int ret =-1;
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_RMDIR, &tmp_err);
 
-	gboolean rmdir_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_RMDIR, terr);	
-	}
-	int rmdir_executor(gfal_plugin_interface* cata_list, GError** terr){
-		return cata_list->rmdirG(cata_list->plugin_data, path, terr);
-	}
+    if(p)
+        ret = p->rmdirG(gfal_get_plugin_handle(p), path,  &tmp_err);
 
-	int ret = gfal_plugins_operation_executor(handle, &rmdir_checker, &rmdir_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret; 	
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 // Execute a opendir function on the appropriate plugin 
@@ -597,19 +551,13 @@ int gfal_plugin_rmdirG(gfal_handle handle, const char* path, GError** err){
 	g_return_val_err_if_fail(handle && name, NULL, err,  "[gfal_plugin_opendir] invalid value");
 	GError* tmp_err=NULL;	
 	gfal_file_handle resu=NULL;
+
+    gfal_plugin_interface* p = gfal_find_plugin(handle, name, GFAL_PLUGIN_OPENDIR, &tmp_err);
+
+    if(p)
+        resu = p->opendirG(gfal_get_plugin_handle(p), name,  &tmp_err);
 	
-	gboolean opendir_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, name, GFAL_PLUGIN_OPENDIR, terr);	
-	}
-	int opendir_executor(gfal_plugin_interface* cata_list, GError** terr){
-		resu= cata_list->opendirG(cata_list->plugin_data, name, terr);
-		return (resu)?0:-1;
-	}
-	
-	gfal_plugins_operation_executor(handle, &opendir_checker, &opendir_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return resu;  
+    G_RETURN_ERR(resu, tmp_err, err);
 }
 
 // Execute a closedir function on the appropriate plugin  
@@ -617,12 +565,10 @@ int gfal_plugin_closedirG(gfal_handle handle, gfal_file_handle fh, GError** err)
 	g_return_val_err_if_fail(handle && fh, -1,err, "[gfal_plugin_closedirG] Invalid args ");	
 	GError* tmp_err=NULL;
 	int ret = -1;
-	gfal_plugin_interface* if_cata = gfal_plugin_getModuleFromHandle(handle, fh, &tmp_err);
+    gfal_plugin_interface* if_cata = gfal_plugin_map_file_handle(handle, fh, &tmp_err);
 	if(!tmp_err)
 		ret = if_cata->closedirG(if_cata->plugin_data, fh, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret;  	
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 // Execute a open function on the appropriate plugin  
@@ -631,19 +577,13 @@ gfal_file_handle gfal_plugin_openG(gfal_handle handle, const char * path, int fl
 	gfal_file_handle resu =NULL;
 	gfal_log(GFAL_VERBOSE_TRACE, " %s ->",__func__);
 
-	
-	gboolean openG_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_OPEN, terr);
-	}	
-	int openG_executor(gfal_plugin_interface* cata_list, GError** terr){
-		resu = cata_list->openG(cata_list->plugin_data, path, flag, mode, terr);
-		return (resu)?0:-1;
-	}	
-	
-	gfal_plugins_operation_executor(handle, &openG_checker, &openG_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);	
-	return resu;
+
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_OPEN, &tmp_err);
+
+    if(p)
+        resu = p->openG(gfal_get_plugin_handle(p), path, flag, mode, &tmp_err);
+
+    G_RETURN_ERR(resu, tmp_err, err);
 }
 
 // Execute a close function on the appropriate plugin  
@@ -651,12 +591,11 @@ int gfal_plugin_closeG(gfal_handle handle, gfal_file_handle fh, GError** err){
 	g_return_val_err_if_fail(handle && fh, -1,err, "[gfal_plugin_closeG] Invalid args ");	
 	GError* tmp_err=NULL;
 	int ret = -1;
-	gfal_plugin_interface* if_cata = gfal_plugin_getModuleFromHandle(handle, fh, &tmp_err);
+    gfal_plugin_interface* if_cata = gfal_plugin_map_file_handle(handle, fh, &tmp_err);
 	if(!tmp_err)
 		ret = if_cata->closeG(if_cata->plugin_data, fh, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret;  	
+
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 // Execute a readdir function on the appropriate plugin  
@@ -664,53 +603,69 @@ struct dirent* gfal_plugin_readdirG(gfal_handle handle, gfal_file_handle fh, GEr
 	g_return_val_err_if_fail(handle && fh, NULL,err, "[gfal_plugin_readdirG] Invalid args ");	
 	GError* tmp_err=NULL;
 	struct dirent* ret = NULL;
-	gfal_plugin_interface* if_cata = gfal_plugin_getModuleFromHandle(handle, fh, &tmp_err);
+    gfal_plugin_interface* if_cata = gfal_plugin_map_file_handle(handle, fh, &tmp_err);
 	if(!tmp_err)
 		ret = if_cata->readdirG(if_cata->plugin_data, fh, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret; 
+
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 
+// Execute a readdir function on the appropriate plugin
+struct dirent* gfal_plugin_readdirppG(gfal_handle handle, gfal_file_handle fh, struct stat* st, GError** err){
+    g_return_val_err_if_fail(handle && fh, NULL,err, "[gfal_plugin_readdirppG] Invalid args ");
+    GError* tmp_err=NULL;
+    struct dirent* ret = NULL;
+    gfal_plugin_interface* if_cata = gfal_plugin_map_file_handle(handle, fh, &tmp_err);
+
+    if(!tmp_err){
+        if(gfal_feature_is_supported(if_cata->readdirppG, g_quark_from_string(GFAL2_PLUGIN_SCOPE) , __func__, &tmp_err))
+            ret = if_cata->readdirppG(if_cata->plugin_data, fh, st, &tmp_err);
+    }
+
+    G_RETURN_ERR(ret, tmp_err, err);
+}
+
+
+
 
 // Execute a getxattr function on the appropriate plugin  
 ssize_t gfal_plugin_getxattrG(gfal_handle handle, const char* path, const char*name, void* buff, size_t s_buff, GError** err){
 	GError* tmp_err=NULL;
 	ssize_t resu = -1;
+
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_GETXATTR, &tmp_err);
+
+    if(p)
+        resu = p->getxattrG(gfal_get_plugin_handle(p), path, name, buff, s_buff, &tmp_err);
 	
-	gboolean getxattr_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_GETXATTR, terr);
-	}	
-	int getxattr_executor(gfal_plugin_interface* cata_list, GError** terr){
-		resu= cata_list->getxattrG(cata_list->plugin_data, path, name, buff, s_buff, terr);
-		return (int)(resu>=0)?0:-1;
-	}
-	
-	gfal_plugins_operation_executor(handle, &getxattr_checker, &getxattr_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);	
-	return resu;	
-	
+    G_RETURN_ERR(resu, tmp_err, err);
 }
 
 // Execute a listxattr function on the appropriate plugin  
 ssize_t gfal_plugin_listxattrG(gfal_handle handle, const char* path, char* list, size_t s_list, GError** err){
 	GError* tmp_err=NULL;
 	ssize_t resu = -1;
-	
-	gboolean listxattr_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_LISTXATTR, terr);
-	}	
-	int listxattr_executor(gfal_plugin_interface* cata_list, GError** terr){
-		resu= cata_list->listxattrG(cata_list->plugin_data, path, list, s_list, terr);
-		return (int)(resu>=0)?0:-1;
-	}
-	
-	gfal_plugins_operation_executor(handle, &listxattr_checker, &listxattr_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);	
-	return resu;		 
+
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_LISTXATTR, &tmp_err);
+
+    if(p)
+        resu = p->listxattrG(gfal_get_plugin_handle(p), path, list, s_list, &tmp_err);
+
+    G_RETURN_ERR(resu, tmp_err, err);
+}
+
+
+// Execute a setxattr function on the appropriate plugin
+int gfal_plugin_setxattrG(gfal_handle handle, const char* path, const char* name, const void* value, size_t size, int flags, GError** err){
+    GError* tmp_err=NULL;
+    int resu = -1;
+
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_SETXATTR, &tmp_err);
+
+    if(p)
+        resu = p->setxattrG(gfal_get_plugin_handle(p), path, name, value, size, flags, &tmp_err);
+    G_RETURN_ERR(resu, tmp_err, err);
 }
 
 
@@ -720,12 +675,10 @@ int gfal_plugin_readG(gfal_handle handle, gfal_file_handle fh, void* buff, size_
 	g_return_val_err_if_fail(handle && fh && buff && s_buff> 0, -1,err, "[gfal_plugin_readG] Invalid args ");	
 	GError* tmp_err=NULL;
 	int ret = -1;
-	gfal_plugin_interface* if_cata = gfal_plugin_getModuleFromHandle(handle, fh, &tmp_err);
+    gfal_plugin_interface* if_cata = gfal_plugin_map_file_handle(handle, fh, &tmp_err);
 	if(!tmp_err)
 		ret = if_cata->readG(if_cata->plugin_data, fh, buff, s_buff,  &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret; 	
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 // Simulate a pread operation in case of non-parallels read support
@@ -745,10 +698,7 @@ inline ssize_t gfal_plugin_simulate_preadG(gfal_handle handle, gfal_plugin_inter
 	}
 	gfal_file_handle_unlock(fh);
 	
-	
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret;
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 // Execute a pread function on the appropriate plugin  
@@ -756,7 +706,7 @@ ssize_t gfal_plugin_preadG(gfal_handle handle, gfal_file_handle fh, void* buff,
 	g_return_val_err_if_fail(handle && fh && buff, -1,err, "[gfal_plugin_preadG] Invalid args ");	
 	GError* tmp_err=NULL;
 	ssize_t ret = -1;
-	gfal_plugin_interface* if_cata = gfal_plugin_getModuleFromHandle(handle, fh, &tmp_err);
+    gfal_plugin_interface* if_cata = gfal_plugin_map_file_handle(handle, fh, &tmp_err);
 	if(!tmp_err){
 		if(if_cata->preadG)
 			ret = if_cata->preadG(if_cata->plugin_data, fh, buff, s_buff, offset,  &tmp_err);
@@ -764,9 +714,7 @@ ssize_t gfal_plugin_preadG(gfal_handle handle, gfal_file_handle fh, void* buff,
 			ret = gfal_plugin_simulate_preadG(handle, if_cata, fh, buff, s_buff, offset, &tmp_err);
 		}	
 	}
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret; 	
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 
@@ -787,10 +735,7 @@ inline ssize_t gfal_plugin_simulate_pwriteG(gfal_handle handle, gfal_plugin_inte
 	}
 	gfal_file_handle_unlock(fh);
 	
-	
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret;
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 
@@ -801,7 +746,7 @@ ssize_t gfal_plugin_pwriteG(gfal_handle handle, gfal_file_handle fh, void* buff,
 	g_return_val_err_if_fail(handle && fh && buff, -1,err, "[gfal_plugin_pwriteG] Invalid args ");	
 	GError* tmp_err=NULL;
 	ssize_t ret = -1;
-	gfal_plugin_interface* if_cata = gfal_plugin_getModuleFromHandle(handle, fh, &tmp_err);
+    gfal_plugin_interface* if_cata = gfal_plugin_map_file_handle(handle, fh, &tmp_err);
 	if(!tmp_err){
 		if(if_cata->pwriteG)
 			ret = if_cata->pwriteG(if_cata->plugin_data, fh, buff, s_buff, offset,  &tmp_err);
@@ -809,9 +754,7 @@ ssize_t gfal_plugin_pwriteG(gfal_handle handle, gfal_file_handle fh, void* buff,
 			ret = gfal_plugin_simulate_pwriteG(handle, if_cata, fh, buff, s_buff, offset, &tmp_err);
 		}	
 	}
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret; 	
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 
@@ -821,12 +764,10 @@ int gfal_plugin_lseekG(gfal_handle handle, gfal_file_handle fh, off_t offset, in
 	g_return_val_err_if_fail(handle && fh , -1,err, "[gfal_plugin_lseekG] Invalid args ");	
 	GError* tmp_err=NULL;
 	int ret = -1;
-	gfal_plugin_interface* if_cata = gfal_plugin_getModuleFromHandle(handle, fh, &tmp_err);
+    gfal_plugin_interface* if_cata = gfal_plugin_map_file_handle(handle, fh, &tmp_err);
 	if(!tmp_err)
 		ret = if_cata->lseekG(if_cata->plugin_data, fh, offset, whence,  &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret;	
+    G_RETURN_ERR(ret, tmp_err, err);
 	
 }
 
@@ -835,12 +776,10 @@ int gfal_plugin_writeG(gfal_handle handle, gfal_file_handle fh, void* buff, size
 	g_return_val_err_if_fail(handle && fh && buff && s_buff> 0, -1,err, "[gfal_plugin_writeG] Invalid args ");	
 	GError* tmp_err=NULL;
 	int ret = -1;
-	gfal_plugin_interface* if_cata = gfal_plugin_getModuleFromHandle(handle, fh, &tmp_err);
+    gfal_plugin_interface* if_cata = gfal_plugin_map_file_handle(handle, fh, &tmp_err);
 	if(!tmp_err)
 		ret = if_cata->writeG(if_cata->plugin_data, fh,buff, s_buff, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);
-	return ret; 	
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 
@@ -848,43 +787,52 @@ int gfal_plugin_writeG(gfal_handle handle, gfal_file_handle fh, void* buff, size
 // Execute a unlink function on the appropriate plugin  
 int gfal_plugin_unlinkG(gfal_handle handle, const char* path, GError** err){
 	GError* tmp_err=NULL;
-	int resu = -1;
-	
-	gboolean unlink_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_UNLINK, terr);
-	}	
-	int unlink_executor(gfal_plugin_interface* cata_list, GError** terr){
-		resu= cata_list->unlinkG(cata_list->plugin_data, path, terr);
-		return resu;
-	}
-	
-	gfal_plugins_operation_executor(handle, &unlink_checker, &unlink_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);	
-	return resu;	
-	
+    int resu = -1;
+    gfal_plugin_interface* p = gfal_find_plugin(handle, path, GFAL_PLUGIN_UNLINK, &tmp_err);
+
+    if(p)
+        resu = p->unlinkG(gfal_get_plugin_handle(p), path,  &tmp_err);
+    G_RETURN_ERR(resu, tmp_err, err);
+
 }
 
-// Execute a setxattr function on the appropriate plugin  
-int gfal_plugin_setxattrG(gfal_handle handle, const char* path, const char* name, const void* value, size_t size, int flags, GError** err){
-	GError* tmp_err=NULL;
-	int resu = -1;
-	
-	gboolean setxattrG_checker(gfal_plugin_interface* cata_list, GError** terr){
-		return gfal_plugin_checker_safe(cata_list, path, GFAL_PLUGIN_SETXATTR, terr);		
-	}
-	int setxattrG_executor(gfal_plugin_interface* cata_list, GError** terr){
-		if(cata_list->setxattrG)
-			return cata_list->setxattrG(cata_list->plugin_data, path, name, value, size, flags, terr);
-		else{
-			g_set_error(terr,0, EPROTONOSUPPORT, "unexcepted NULL pointer for the setxattrG call of the %s plugin", cata_list->getName());
-			return -1;
-		}
-	}
+int gfal_plugin_bring_onlineG(gfal2_context_t handle, const char* uri,
+                              time_t pintime, time_t timeout,
+                              char* token, size_t tsize,
+                              int async,
+                              GError ** err){
+    GError* tmp_err=NULL;
+    int resu = -1;
+    gfal_plugin_interface* p = gfal_find_plugin(handle, uri, GFAL_PLUGIN_BRING_ONLINE, &tmp_err);
 
-	resu = gfal_plugins_operation_executor(handle, &setxattrG_checker, &setxattrG_executor, &tmp_err);
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]",__func__);	
-	return resu;		
+    if(p)
+        resu = p->bring_online(gfal_get_plugin_handle(p), uri,
+                                 pintime, timeout,
+                                 token, tsize,
+                                 async,
+                                 &tmp_err);
+    G_RETURN_ERR(resu, tmp_err, err);
+ }
+
+
+int gfal_plugin_bring_online_pollG(gfal2_context_t handle, const char* uri,
+                                   const char* token, GError ** err) {
+    GError* tmp_err=NULL;
+    int resu = -1;
+    gfal_plugin_interface* p = gfal_find_plugin(handle, uri, GFAL_PLUGIN_BRING_ONLINE, &tmp_err);
+
+    if(p)
+        resu = p->bring_online_poll(gfal_get_plugin_handle(p), uri,  token, &tmp_err);
+    G_RETURN_ERR(resu, tmp_err, err);
 }
 
+int gfal_plugin_release_fileG(gfal2_context_t handle, const char* uri,
+                              const char* token, GError ** err) {
+    GError* tmp_err=NULL;
+    int resu = -1;
+    gfal_plugin_interface* p = gfal_find_plugin(handle, uri, GFAL_PLUGIN_BRING_ONLINE, &tmp_err);
+
+    if(p)
+        resu = p->release_file(gfal_get_plugin_handle(p), uri,  token, &tmp_err);
+    G_RETURN_ERR(resu, tmp_err, err);
+}
