@@ -45,11 +45,14 @@ extern "C"
 
 /**
  * Prototype of the plugins entry point
+ *
+ *  return gfal_plugin_interface must be allocated with \êef gfal_plugin_interface_new
+ *
  * @param handle : gfal_handle of the current call
  * @param err : Error report in case of fatal error while the plugin load.
  * 
  * */
-typedef gfal_plugin_interface (*gfal_plugin_init_t)(gfal_handle handle, GError** err);
+typedef gfal_plugin_interface* (*gfal_plugin_init_t)(gfal_handle handle, GError** err);
 
 
 /**
@@ -61,14 +64,14 @@ typedef gfal_plugin_interface (*gfal_plugin_init_t)(gfal_handle handle, GError**
  */
 struct _gfal_plugin_interface{
 	 //! @cond
-	 // internal gfal data : touching this causes the death of a little cat
+     // internal gfal data : touching this triggers the death of a kitty
 	void * gfal_data;
 	//! @endcond
 	
     // plugin management
 
 	/**
-	 *  plugin reserved pointer, free to use for plugin's personnal datas
+     *  plugin reserved pointer, free to use for plugin's internal data, passed to any function
 	 * 
 	 * */
 	plugin_handle plugin_data;
@@ -225,11 +228,14 @@ struct _gfal_plugin_interface{
 	 * 			
 	 *  @param plugin_data : internal plugin data
 	 *  @param url : url of the directory to create
+     *  @param mode : right mode of the created directory
+     *  @param rec_flag : recursive mode, if enabled the plugin MUST create the parent directories if needed,
+     *       if the rec_flag is not supported by this plugin, the plugin MUST return a negative value and set the GError errcode to ENOENT
 	 *  @param err : Error report, the code field of err should be set to errno value when possible
 	 *  @return 0 in case of success or -1 if error occures,
 	 *          err MUST be set in case of error
 	 * */	
-	int (*mkdirpG)(plugin_handle plugin_data, const char* url, mode_t, gboolean pflag, GError** err);  //function pointer for the mkdir call, if pflag is set, the call is considered like a recursive call for a full path creation
+    int (*mkdirpG)(plugin_handle plugin_data, const char* url, mode_t mode, gboolean rec_flag, GError** err);
 	/**
 	 *  OPTIONAL : gfal_rmdir function  support
 	 * 			
@@ -394,15 +400,70 @@ struct _gfal_plugin_interface{
        *
     */
      int (*copy_file)(plugin_handle plugin_data, gfal2_context_t context, gfalt_params_t params, const char* src, const char* dst, GError** );
+
+     /**
+      *
+      * OPTIONAL: Requests to stage a file to the fist layer on a hierarchical SE.
+      * @param plugin_data : internal plugin context
+      * @param url : The url of the file
+      * @param pintime : Time the file should stay in the cache
+      * @param timeout : Operation timeout
+      * @param token Where to put the retrieved token.
+      * @param tsize The size of the buffer pointed by token.
+      * @param async If true (!= 0), the call will not block. The caller will need
+      *              to use bring_online_poll later.
+      * @param err:  GError error support
+      * @return      -1 on error (and err is set). 0 on success. 1 if the file has been staged.
+      */
+     int (*bring_online)(plugin_handle plugin_data, const char* url,
+                         time_t pintime, time_t timeout,
+                         char* token, size_t tsize,
+                         int async,
+                         GError** err);
+
+     /**
+      * OPTIONAL: Polling the bring_online request (mandatory if bring online is supported)
+      * @param url   The same URL as was passed to bring_online_async
+      * @param token The token as returned by bring_online_async
+      * @return      -1 on error (and err is set). 0 on success. 1 if the file has been staged.
+      */
+     int (*bring_online_poll)(plugin_handle plugin_data, const char* url,
+                              const char* token, GError** err);
+
+     /**
+      * OPTIONAL: Releases a previously staged file (mandatory if bring online is supported)
+      * @param plugin_data : internal plugin context
+      * @param url :  The url of the file
+      * @param token: The request token. If NULL,
+      * @param err:   GError error support
+      */
+     int (*release_file)(plugin_handle plugin_data, const char* url,
+                         const char* token,
+                         GError** err);
+
+     /**
+      *  OPTIONAL : gfal_readdirpp function support
+      *             Allow directory listing + get meta-data in one operation
+      *
+      *  @param plugin_data : internal plugin data
+      *  @param dir_desc : directory descriptor to use
+      *  @param st : struct stat to fill
+      *  @param err : Error report, the code field of err should be set to errno value when possible
+      *  @return dirent information in case of success or NULL if end of listing or error,
+      *          err MUST be set in case of error
+      * */
+     struct dirent* (*readdirppG)(plugin_handle plugin_data, gfal_file_handle dir_desc, struct stat* st, GError** err);
+
 	 // reserved for future usage
 	 //! @cond
-     void* future[24];
+     void* future[21];
 	 //! @endcond
 };
 
 //! @cond
 struct _plugin_opts{
 	gfal_plugin_interface plugin_list[MAX_PLUGIN_LIST];
+    GList* sorted_plugin;
 	int plugin_number;
 };
 //! @endcond
@@ -426,6 +487,7 @@ int gfal_plugin_mkdirp(gfal_handle handle, const char* path, mode_t mode, gboole
 
 
 gfal_file_handle gfal_plugin_opendirG(gfal_handle handle, const char* name, GError** err);
+struct dirent* gfal_plugin_readdirppG(gfal_handle handle, gfal_file_handle fh, struct stat* st, GError** err);
 int gfal_plugin_closedirG(gfal_handle handle, gfal_file_handle fh, GError** err);
 struct dirent* gfal_plugin_readdirG(gfal_handle handle, gfal_file_handle fh, GError** err);
  	
@@ -446,6 +508,19 @@ int gfal_plugin_unlinkG(gfal_handle handle, const char* path, GError** err);
 ssize_t gfal_plugin_getxattrG(gfal_handle, const char*, const char*, void* buff, size_t s_buff, GError** err);
 ssize_t gfal_plugin_listxattrG(gfal_handle, const char*, char* list, size_t s_list, GError** err);
 int gfal_plugin_setxattrG(gfal_handle, const char*, const char*, const void*, size_t, int, GError**);
+
+int gfal_plugin_bring_onlineG(gfal2_context_t handle, const char* uri,
+                              time_t pintime, time_t timeout,
+                              char* token, size_t tsize,
+                              int async,
+                              GError ** err);
+
+int gfal_plugin_bring_online_pollG(gfal2_context_t handle, const char* uri,
+                                   const char* token, GError ** err);
+
+int gfal_plugin_release_fileG(gfal2_context_t handle, const char* uri,
+                              const char* token, GError ** err);
+
 //! @endcond
 
 #ifdef __cplusplus
