@@ -33,19 +33,13 @@
 #include "gridftpinterface.h"
 #include "gridftpwrapper.h"
 
-#include "gridftp_ifce_filecopy.h"
-#include "gridftp_stat_module.h"
-#include "gridftp_mkdir_module.h"
-#include "gridftp_chmod_module.h"
-#include "gridftp_rmdir_module.h"
-#include "gridftp_opendir_module.h"
-#include "gridftp_unlink_module.h"
-#include "gridftp_rw_module.h"
-#include "gridftp_checksum_module.h"
+#include "gridftp_filecopy.h"
+#include "gridftp_namespace.h"
 
 typedef globus_gass_copy_glob_stat_t gfal_globus_stat_t;
 
 
+
 class GridftpModule 
 {
 	public:
@@ -88,6 +82,8 @@ class GridftpModule
                                char * checksum_buffer, size_t buffer_length,
                                off_t start_offset, size_t data_length);
 
+        // Rename
+        virtual void rename(const char* src, const char* dst);
 		
 		// rmdir query on path
 	    virtual void rmdir(const char* path);	
@@ -104,7 +100,7 @@ class GridftpModule
         void autoCleanFileCopy(gfalt_params_t params, GError* checked_error, const char* dst);
 
 		 // Execute a file transfer operation for gridftp URLs
-		virtual int filecopy(gfalt_params_t params, const char* src, const char* dst);
+		virtual void filecopy(gfalt_params_t params, const char* src, const char* dst);
 		
 
 				 
