@@ -22,6 +22,7 @@ const char* gridftp_version_config = "GRIDFTP_V2";
 const char* gridftp_session_reuse_config= "SESSION_REUSE";
 const char* gridftp_dcau_config= "DCAU";
 const char* gridftp_ipv6_config= "IPV6";
+const char* gridftp_delay_passv_config = "DELAY_PASSV";
 
 struct RwStatus{
 	off_t init;
@@ -53,6 +54,28 @@ struct Gass_attr_handler_implem : public Gass_attr_handler{
 };
 
 struct GridFTP_session_implem : public GridFTP_session{
+
+    GridFTP_session_implem(GridFTPFactory* f, const std::string & thostname) :
+        factory(f),
+        hostname(thostname){
+        init();
+    }
+
+    GridFTP_session_implem( GridFTP_session_implem *src) :
+    factory(src->factory),
+    hostname(src->hostname),
+      _sess(src->_sess)
+    {   }
+
+    virtual ~GridFTP_session_implem(){
+        if(_sess != NULL){
+            clean();
+            if(_isDirty)
+                this->purge();
+            else
+                factory->gfal_globus_ftp_release_handle_internal(this);
+        }
+    }
 	
 	void init(){
 		_sess = new Session_handler();
@@ -118,6 +141,10 @@ struct GridFTP_session_implem : public GridFTP_session{
         globus_ftp_client_operationattr_set_allow_ipv6(&(_sess->operation_attr_ftp), (globus_bool_t) enable);
     }
 
+    void set_delayed_pass(bool enable){
+        globus_ftp_client_operationattr_set_delayed_pasv(&(_sess->operation_attr_ftp),  (globus_bool_t) (enable)?GLOBUS_TRUE:GLOBUS_FALSE);
+    }
+
     void set_dcau(const globus_ftp_control_dcau_t & _dcau ){
         _sess->dcau.mode = _dcau.mode;
         globus_ftp_client_operationattr_set_dcau(&(_sess->operation_attr_ftp), &(_sess->dcau));
@@ -143,29 +170,6 @@ struct GridFTP_session_implem : public GridFTP_session{
             _sess->tcp_buffer_size.fixed.size = tcp_buffer_size;
         }
     }
-
-	GridFTP_session_implem(GridFTPFactory* f, const std::string & hostname){
-		this->factory = f;
-		this->hostname = hostname;
-		init();	
-	}
-	
-	GridFTP_session_implem( GridFTP_session_implem *src){
-		this->factory = src->factory;
-		this->hostname = src->hostname;
-		this->_sess = src->_sess;
-
-	}
-	
-	virtual ~GridFTP_session_implem(){
-        if(_sess != NULL){
-            clean();
-            if(_isDirty)
-                this->purge();
-            else
-                factory->gfal_globus_ftp_release_handle_internal(this);
-        }
-	}
 	
 	virtual globus_ftp_client_handle_t* get_ftp_handle(){
 		globus_result_t res = globus_gass_copy_get_ftp_handle(&(_sess->gass_handle), &(_sess->handle_ftp));
@@ -225,10 +229,13 @@ struct GridFTP_session_implem : public GridFTP_session{
         globus_ftp_control_tcpbuffer_t tcp_buffer_size;
 	};
 	
-	Session_handler* _sess;
+
 	// internal fields	
+    GridFTPFactory* factory;
 	std::string hostname;		
-	GridFTPFactory* factory;
+
+    // sess
+    Session_handler* _sess;
 };
 
 
@@ -244,7 +251,7 @@ GridFTPFactory::GridFTPFactory(gfal_handle handle) : _handle(handle)
 
 
 
-GridFTP_Request_state::GridFTP_Request_state(GridFTP_session * s, bool own_session, GridFtp_request_type request_type) : sess(s), end_time(0,0){
+GridFTP_Request_state::GridFTP_Request_state(GridFTP_session * s, bool own_session, GridFtp_request_type request_type) : errcode(0), sess(s), end_time(0,0){
 	req_status=GRIDFTP_REQUEST_NOT_LAUNCHED;
     this->own_session = own_session;
     this->request_type = request_type;
@@ -311,11 +318,11 @@ GridFTP_session* GridFTPFactory::get_recycled_handle(const std::string & hostnam
     GridFTP_session* res= NULL;
     std::multimap<std::string, GridFTP_session*>::iterator it=sess_cache.find(hostname); // try to find a session explicitely associated with this handle
     if(it == sess_cache.end()){ // if no session found, take a generic one
-        gfal_log(GFAL_VERBOSE_TRACE, "recycled unamed generic session found .... ");
+        gfal_log(GFAL_VERBOSE_TRACE, "no session associated with this hostname, try find generic one .... ");
         it = sess_cache.begin();
     }
     if(it != sess_cache.end()){
-        gfal_log(GFAL_VERBOSE_TRACE, "gridftp session for %s found in  cache !", hostname.c_str());
+        gfal_log(GFAL_VERBOSE_TRACE, "gridftp session for: %s found in  cache !", hostname.c_str());
         res = (*it).second;
         sess_cache.erase(it);
     }else{
@@ -344,7 +351,7 @@ static int scan_errstring(const char *p) {
     int ret = ECOMM;
     if (p == NULL) return ret;
 
-    if (strstr(p, "o such file") || strstr(p, "File not found"))
+    if (strstr(p, "o such file") || strstr(p, "not found"))
         ret = ENOENT;
     else if (strstr(p, "ermission denied") || strstr(p, "credential"))
         ret = EACCES;
@@ -408,7 +415,8 @@ GridFTP_session* GridFTPFactory::get_new_handle(const std::string & hostname){
     if(tmp_err)
         throw Glib::Error(tmp_err);
 
-    bool ipv6 = gfal2_get_opt_boolean_with_default(_handle, GRIDFTP_CONFIG_GROUP, gridftp_ipv6_config, false);
+    const bool ipv6 = gfal2_get_opt_boolean_with_default(_handle, GRIDFTP_CONFIG_GROUP, gridftp_ipv6_config, false);
+    const bool delay_passv = gfal2_get_opt_boolean_with_default(_handle, GRIDFTP_CONFIG_GROUP, gridftp_delay_passv_config, true);
 
     dcau_param.mode = (gfal2_get_opt_boolean(_handle, GRIDFTP_CONFIG_GROUP, gridftp_dcau_config, &tmp_err))?GLOBUS_FTP_CONTROL_DCAU_DEFAULT:GLOBUS_FTP_CONTROL_DCAU_NONE;
     if(tmp_err)
@@ -419,6 +427,7 @@ GridFTP_session* GridFTPFactory::get_new_handle(const std::string & hostname){
 	sess->set_gridftpv2(gridftp_v2);
     sess->set_dcau(dcau_param);
     sess->set_ipv6(ipv6);
+    sess->set_delayed_pass(delay_passv);
 	return sess.release();
 }
 
@@ -515,7 +524,7 @@ void GridFTP_Request_state::poll_callback(const Glib::Quark &scope){
     }
 
     if(timeout && this->canceling==FALSE){
-       gfal_log(GFAL_VERBOSE_TRACE,"gfal gridftp operation timeout occures ! cancel the operation ...");
+       gfal_log(GFAL_VERBOSE_TRACE,"gfal gridftp operation timeout occurred ! cancel the operation ...");
        cancel_operation(scope, "gfal gridftp internal operation timeout, operation canceled");
        this->set_error_code(ETIMEDOUT);
     }
@@ -529,7 +538,17 @@ void GridFTP_Request_state::err_report(const Glib::Quark &scope){
 }
 
 
-void GridFTP_Request_state::wait_callback(const Glib::Quark &scope){
+void GridFTP_Request_state::wait_callback(const Glib::Quark &scope, time_t timeout){
+    struct timespec st_timeout;
+
+    st_timeout.tv_nsec = 0;
+    st_timeout.tv_sec  = timeout;
+
+    gfal_log(GFAL_VERBOSE_TRACE,
+             "   [GridFTP_Request_state::wait_callback] setup gsiftp timeout to %ld seconds",
+             timeout);
+
+    init_timeout(&st_timeout);
     poll_callback(scope);
     err_report(scope);
 }
@@ -704,7 +723,8 @@ ssize_t gridftp_write_stream(const Glib::Quark & scope,
 
 std::string gridftp_hostname_from_url(const char * url){
 	GError * tmp_err=NULL;
-	char buffer[GFAL_URL_MAX_LEN];
+    char buffer[GFAL_URL_MAX_LEN];
+    buffer[0] ='\0';
 	const int res = gfal_hostname_from_uri(url, buffer, GFAL_URL_MAX_LEN, &tmp_err);
 	if(res < 0)
 		throw Glib::Error(tmp_err);
