@@ -20,6 +20,8 @@
 #include <glib.h>
 #include <cancel/gfal_cancel.h>
 #include "gridftpmodule.h"
+#include "gridftpinterface.h"
+#include <algorithm>
 
 
 #define GRIDFTP_CONFIG_GROUP "GRIDFTP PLUGIN"
@@ -35,6 +37,74 @@ enum GridFtp_request_type{
     GRIDFTP_REQUEST_FTP
 };
 
+struct Uri
+{
+public:
+std::string QueryString, Path, Protocol, Host, Port;
+
+static Uri Parse(const std::string &uri)
+{
+    Uri result;
+
+    typedef std::string::const_iterator iterator_t;
+
+    if (uri.length() == 0)
+        return result;
+
+    iterator_t uriEnd = uri.end();
+
+    // get query start
+    iterator_t queryStart = std::find(uri.begin(), uriEnd, '?');
+
+    // protocol
+    iterator_t protocolStart = uri.begin();
+    iterator_t protocolEnd = std::find(protocolStart, uriEnd, ':');            //"://");
+
+    if (protocolEnd != uriEnd)
+    {
+        std::string prot = &*(protocolEnd);
+        if ((prot.length() > 3) && (prot.substr(0, 3) == "://"))
+        {
+            result.Protocol = std::string(protocolStart, protocolEnd);
+            protocolEnd += 3;   //      ://
+        }
+        else
+            protocolEnd = uri.begin();  // no protocol
+    }
+    else
+        protocolEnd = uri.begin();  // no protocol
+
+    // host
+    iterator_t hostStart = protocolEnd;
+    iterator_t pathStart = std::find(hostStart, uriEnd, '/');  // get pathStart
+
+    iterator_t hostEnd = std::find(protocolEnd, 
+        (pathStart != uriEnd) ? pathStart : queryStart, ':');  // check for port
+
+    result.Host = std::string(hostStart, hostEnd);
+
+    // port
+    if ((hostEnd != uriEnd) && ((&*(hostEnd))[0] == ':'))  // we have a port
+    {
+        hostEnd++;
+        iterator_t portEnd = (pathStart != uriEnd) ? pathStart : queryStart;
+        result.Port = std::string(hostEnd, portEnd);
+    }
+
+    // path
+    if (pathStart != uriEnd)
+        result.Path = std::string(pathStart, queryStart);
+
+    // query
+    if (queryStart != uriEnd)
+        result.QueryString = std::string(queryStart, uri.end());
+
+    return result;
+
+}   // Parse
+};  // uri
+
+
 struct GridFTP_Request_state{
  protected:
    Glib::Mutex internal_lock;
@@ -44,6 +114,26 @@ struct GridFTP_Request_state{
    
    Gridftp_request_status req_status;
    
+   inline void init_timeout(struct timespec * time_offset){
+       if( time_offset  && timespec_isset(time_offset)){
+           end_time.assign_current_time();
+           end_time.add_seconds(time_offset->tv_sec);
+           end_time.add_microseconds(time_offset->tv_nsec/1000);
+       }else{
+         end_time = Glib::TimeVal(0,0);
+       }
+   }
+
+   inline void set_end_time(struct timespec * my_end_time){
+       if(my_end_time && timespec_isset(my_end_time)){
+           end_time = Glib::TimeVal(my_end_time->tv_sec, my_end_time->tv_nsec/1000);
+       }else{
+         end_time = Glib::TimeVal(0,0);
+       }
+   }
+
+   void poll_callback(const Glib::Quark & scope);
+
  public:
     GridFTP_Request_state(GridFTP_session * s, bool own_session=true,  GridFtp_request_type request_type = GRIDFTP_REQUEST_FTP);
 	virtual ~GridFTP_Request_state();
@@ -68,25 +158,6 @@ struct GridFTP_Request_state{
     inline void start(){
         this->req_status = GRIDFTP_REQUEST_RUNNING;
     }
-
-    //
-    inline void init_timeout(struct timespec * time_offset){
-        if( time_offset  && timespec_isset(time_offset)){
-            end_time.assign_current_time();
-            end_time.add_seconds(time_offset->tv_sec);
-            end_time.add_microseconds(time_offset->tv_nsec/1000);
-        }else{
-          end_time = Glib::TimeVal(0,0);
-        }
-    }
-
-    inline void set_end_time(struct timespec * my_end_time){
-        if(my_end_time && timespec_isset(my_end_time)){
-            end_time = Glib::TimeVal(my_end_time->tv_sec, my_end_time->tv_nsec/1000);
-        }else{
-          end_time = Glib::TimeVal(0,0);
-        }
-    }
     
     inline int get_error_code(void) {
       Glib::Mutex::Lock locker(internal_lock);
@@ -118,8 +189,8 @@ struct GridFTP_Request_state{
       req_status = st;
     }
 
-    void poll_callback(const Glib::Quark & scope);
-    void wait_callback(const Glib::Quark & scope);
+    void wait_callback(const Glib::Quark & scope, time_t timeout = 300);
+
     void err_report(const Glib::Quark &scope);
 
     void cancel_operation(const Glib::Quark &scope, const std::string & msg = "");
