@@ -16,7 +16,7 @@
 */
 
  
-#define _GNU_SOURCE 
+ 
 
 #include <regex.h>
 #include <time.h> 
@@ -46,7 +46,7 @@ static int gfal_srm_convert_filestatuses_to_srm_result(struct srmv2_pinfilestatu
 		if(statuses[i].explanation)
 			g_strlcpy((*resu)[i].err_str, statuses[i].explanation, GFAL_URL_MAX_LEN);
 		(*resu)[i].err_code = statuses[i].status;	
-		(*resu)[i].reqtoken = reqtoken;
+		(*resu)[i].reqtoken = g_strdup(reqtoken);
 	}
 	return 0;
 }
@@ -56,19 +56,24 @@ int gfal_srmv2_get_global(gfal_srm_plugin_t handle, gfal_srm_params_t params, st
 	g_return_val_err_if_fail(handle != NULL && input != NULL && resu != NULL,-1,err,"[gfal_srmv2_get_global] tab null ");
 
 	GError* tmp_err=NULL;
-	char errbuf[GFAL_URL_MAX_LEN];
-	errbuf[0]='\0';
 	int ret=0;
 	struct srm_preparetoget_output preparetoget_output;
+    
+    memset(&preparetoget_output, 0, sizeof(preparetoget_output));
 	
 	ret = gfal_srm_external_call.srm_prepare_to_get(context,input,&preparetoget_output);
 	if(ret < 0){
-		gfal_srm_report_error(errbuf, &tmp_err);
+		gfal_srm_report_error(context->errbuf, &tmp_err);
 	} else{
 		gfal_srm_convert_filestatuses_to_srm_result(preparetoget_output.filestatuses, preparetoget_output.token, ret, resu,  &tmp_err);
-    	gfal_srm_external_call.srm_srmv2_pinfilestatus_delete(preparetoget_output.filestatuses, ret);
-    	gfal_srm_external_call.srm_srm2__TReturnStatus_delete(preparetoget_output.retstatus);
 	}
+    
+    if (preparetoget_output.filestatuses != NULL)
+        gfal_srm_external_call.srm_srmv2_pinfilestatus_delete(preparetoget_output.filestatuses, ret);
+    if (preparetoget_output.retstatus != NULL)
+        gfal_srm_external_call.srm_srm2__TReturnStatus_delete(preparetoget_output.retstatus);
+    free(preparetoget_output.token);
+    
 	G_RETURN_ERR(ret, tmp_err, err);		
 }
 
@@ -78,19 +83,24 @@ int gfal_srmv2_put_global(gfal_srm_plugin_t handle, gfal_srm_params_t params, st
 	g_return_val_err_if_fail(handle != NULL && input != NULL && resu != NULL,-1,err,"[gfal_srmv2_put_global] tab null ");
 
 	GError* tmp_err=NULL;
-	char errbuf[GFAL_URL_MAX_LEN];
-	errbuf[0]='\0';
 	int ret=0;
 	struct srm_preparetoput_output preparetoput_output;
+    
+    memset(&preparetoput_output, 0, sizeof(preparetoput_output));
 	
 	ret = gfal_srm_external_call.srm_prepare_to_put(context, input, &preparetoput_output);
 	if(ret < 0){
-		gfal_srm_report_error(errbuf, &tmp_err);
+		gfal_srm_report_error(context->errbuf, &tmp_err);
 	} else{
 		gfal_srm_convert_filestatuses_to_srm_result(preparetoput_output.filestatuses, preparetoput_output.token, ret, resu, &tmp_err);
-    	gfal_srm_external_call.srm_srmv2_pinfilestatus_delete(preparetoput_output.filestatuses, ret);
-    	gfal_srm_external_call.srm_srm2__TReturnStatus_delete(preparetoput_output.retstatus);
 	}
+    
+    if (preparetoput_output.filestatuses != NULL)
+        gfal_srm_external_call.srm_srmv2_pinfilestatus_delete(preparetoput_output.filestatuses, ret);
+    if (preparetoput_output.retstatus != NULL)
+        gfal_srm_external_call.srm_srm2__TReturnStatus_delete(preparetoput_output.retstatus);
+    free(preparetoput_output.token);
+    
 	G_RETURN_ERR(ret, tmp_err, err);		
 }
 
@@ -99,8 +109,7 @@ int gfal_srm_getTURLS_srmv2_internal(gfal_srmv2_opt* opts,  gfal_srm_params_t pa
 	g_return_val_err_if_fail(surls!=NULL,-1,err,"[gfal_srmv2_getasync] tab null ");
 			
 	GError* tmp_err=NULL;
-	struct srm_context context;
-	int ret=0;
+    int ret=-1;
 	struct srm_preparetoget_input preparetoget_input;
 	const int err_size = 2048;
 	
@@ -113,10 +122,10 @@ int gfal_srm_getTURLS_srmv2_internal(gfal_srmv2_opt* opts,  gfal_srm_params_t pa
 	preparetoget_input.protocols = gfal_srm_params_get_protocols(params);
     preparetoget_input.spacetokendesc = gfal_srm_params_get_spacetoken(params);
 	preparetoget_input.surls = surls;	
-    gfal_srm_ifce_context_init(&context, opts->handle, endpoint,
-                                  errbuf, err_size, &tmp_err);
-	
-	ret = gfal_srmv2_get_global(opts, params, &context, &preparetoget_input, resu, &tmp_err);
+    srm_context_t context = gfal_srm_ifce_context_setup(opts->handle, endpoint, errbuf, err_size, &tmp_err);
+    if(context)
+        ret = gfal_srmv2_get_global(opts, params, context, &preparetoget_input, resu, &tmp_err);
+    gfal_srm_ifce_context_release(context);
 	G_RETURN_ERR(ret, tmp_err, err);
 }
 
@@ -125,17 +134,15 @@ int gfal_srm_putTURLS_srmv2_internal(gfal_srmv2_opt* opts , gfal_srm_params_t pa
 	g_return_val_err_if_fail(surls!=NULL,-1,err,"[gfal_srm_putTURLS_srmv2_internal] GList passed null");
 			
 	GError* tmp_err=NULL;
-	struct srm_context context;
-	int ret=0,i=0;
+    int ret=-1,i=0;
 	struct srm_preparetoput_input preparetoput_input;
 	const int err_size = 2048;
-
 	
 	char errbuf[err_size]; errbuf[0]='\0';
 	size_t n_surl = g_strv_length (surls);									// n of surls
-	gint64 filesize_tab[n_surl];
+	SRM_LONG64 filesize_tab[n_surl];
 	for(i=0; i < n_surl;++i)
-		filesize_tab[i] = opts->filesizes;
+        filesize_tab[i] = params->file_size;
 		
 	// set the structures datafields	
 	preparetoput_input.desiredpintime = opts->opt_srmv2_desiredpintime;		
@@ -144,11 +151,11 @@ int gfal_srm_putTURLS_srmv2_internal(gfal_srmv2_opt* opts , gfal_srm_params_t pa
     preparetoput_input.spacetokendesc = gfal_srm_params_get_spacetoken(params);
 	preparetoput_input.surls = surls;	
 	preparetoput_input.filesizes = filesize_tab;
-    gfal_srm_ifce_context_init(&context, opts->handle, endpoint,
-                                  errbuf, err_size, &tmp_err);
 	
-	
-	ret = gfal_srmv2_put_global(opts, params, &context, &preparetoput_input, resu, &tmp_err);
+    srm_context_t context = gfal_srm_ifce_context_setup(opts->handle, endpoint, errbuf, err_size, &tmp_err);
+    if(context)
+        ret = gfal_srmv2_put_global(opts, params, context, &preparetoput_input, resu, &tmp_err);
+    gfal_srm_ifce_context_release(context);
 	G_RETURN_ERR(ret, tmp_err, err);	
 }
 
@@ -162,7 +169,6 @@ int gfal_srm_mTURLS_internal(gfal_srmv2_opt* opts, gfal_srm_params_t params, srm
 	enum gfal_srm_proto srm_types;
 
 	if((gfal_srm_determine_endpoint(opts, *surls, full_endpoint, GFAL_URL_MAX_LEN, &srm_types, &tmp_err)) == 0){		// check & get endpoint										
-		gfal_log(GFAL_VERBOSE_NORMAL, "[gfal_srm_mTURLS_internal] endpoint %s", full_endpoint);
 
 		if (srm_types == PROTO_SRMv2){
 			if(req_type == SRM_GET)
@@ -176,9 +182,7 @@ int gfal_srm_mTURLS_internal(gfal_srmv2_opt* opts, gfal_srm_params_t params, srm
 		}		
 	}
 
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-	return ret;		
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 //  simple wrapper to getTURLs for the gfal_module layer
@@ -201,14 +205,13 @@ int gfal_srm_getTURLS_plugin(plugin_handle ch, const char* surl, char* buff_turl
 			}else{
 				g_set_error(&tmp_err,0 , resu[0].err_code, " error on the turl request : %s ", resu[0].err_str);
 				ret = -1;
+                g_free(resu->reqtoken);
 			}
 			free(resu);
 		}
 		gfal_srm_params_free(params);
 	}	
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-	return ret;			
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 
@@ -221,7 +224,7 @@ int gfal_srm_getTURL_checksum(plugin_handle ch, const char* surl, char* buff_tur
     int ret = -1;
 
     gfal_srm_params_t params = gfal_srm_params_new(opts, & tmp_err);
-    gfal_srm_params_set_protocols(params, opts->opt_srmv2_tp3_protocols);
+    gfal_srm_params_set_protocols(params, srm_get_3rdparty_turls_sup_protocol(opts->handle));
     if(params != NULL){
         ret= gfal_srm_mTURLS_internal(opts, params, SRM_GET, surls, &resu, &tmp_err);
         if(ret >0){
@@ -236,9 +239,7 @@ int gfal_srm_getTURL_checksum(plugin_handle ch, const char* surl, char* buff_tur
         }
         gfal_srm_params_free(params);
     }
-    if(tmp_err)
-        g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-    return ret;
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 
@@ -254,7 +255,7 @@ int gfal_srm_get_rd3_turl(plugin_handle ch, gfalt_params_t p, const char* surl,
 	
 	gfal_srm_params_t params = gfal_srm_params_new(opts, & tmp_err);
     gfal_srm_params_set_spacetoken(params, gfalt_get_src_spacetoken(p, NULL));
-	gfal_srm_params_set_protocols(params, opts->opt_srmv2_tp3_protocols);
+    gfal_srm_params_set_protocols(params, srm_get_3rdparty_turls_sup_protocol(opts->handle));
 	if(params != NULL){
 	ret= gfal_srm_mTURLS_internal(opts, params, SRM_GET, surls, &resu, &tmp_err);
 		if(ret >=0){
@@ -294,7 +295,7 @@ int gfal_srm_getTURLS(gfal_srmv2_opt* opts, char** surls, gfal_srm_result** resu
 
 
 //  execute a put for thirdparty transfer turl
-int gfal_srm_put_rd3_turl(plugin_handle ch,  gfalt_params_t p, const char* surl, char* buff_turl, int size_turl, char** reqtoken, GError** err){
+int gfal_srm_put_rd3_turl(plugin_handle ch,  gfalt_params_t p, const char* surl, size_t surl_file_size, char* buff_turl, int size_turl, char** reqtoken, GError** err){
 	gfal_srmv2_opt* opts = (gfal_srmv2_opt*)ch;
 	gfal_srm_result* resu=NULL;
 	GError* tmp_err=NULL;
@@ -303,7 +304,8 @@ int gfal_srm_put_rd3_turl(plugin_handle ch,  gfalt_params_t p, const char* surl,
 	
 	gfal_srm_params_t params = gfal_srm_params_new(opts, & tmp_err);
     gfal_srm_params_set_spacetoken(params, gfalt_get_dst_spacetoken(p, NULL));
-	gfal_srm_params_set_protocols(params, opts->opt_srmv2_tp3_protocols);	
+    gfal_srm_params_set_protocols(params, srm_get_3rdparty_turls_sup_protocol(opts->handle));
+    gfal_srm_params_set_size(params, surl_file_size);
 	if(params != NULL){
 		ret= gfal_srm_mTURLS_internal(opts, params, SRM_PUT, surls, &resu,  &tmp_err);
 		if(ret >=0){
@@ -321,9 +323,7 @@ int gfal_srm_put_rd3_turl(plugin_handle ch,  gfalt_params_t p, const char* surl,
 		gfal_srm_params_free(params);
 	}
 	
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-	return ret;			
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 
@@ -348,14 +348,12 @@ int gfal_srm_putTURLS_plugin(plugin_handle ch, const char* surl, char* buff_turl
 				g_set_error(&tmp_err,0 , resu[0].err_code, " error on the turl request : %s ", resu[0].err_str);
 				ret = -1;
 			}
-		
+            free(resu);
 		}
 		gfal_srm_params_free(params);
 	}
 	
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-	return ret;			
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 
@@ -373,9 +371,7 @@ int gfal_srm_putTURLS(gfal_srmv2_opt* opts , char** surls, gfal_srm_result** res
 		}
 		gfal_srm_params_free(params);
 	}
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-	return ret;	
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 
@@ -384,7 +380,6 @@ static int gfal_srm_putdone_srmv2_internal(gfal_srmv2_opt* opts, char* endpoint,
 	g_return_val_err_if_fail(surls!=NULL,-1,err,"[gfal_srm_putdone_srmv2_internal] invalid args ");
 			
 	GError* tmp_err=NULL;
-	struct srm_context context;
 	int ret=0;
 	struct srm_putdone_input putdone_input;
 	struct srmv2_filestatus *statuses;
@@ -398,19 +393,20 @@ static int gfal_srm_putdone_srmv2_internal(gfal_srmv2_opt* opts, char* endpoint,
 	putdone_input.reqtoken = token;
 	putdone_input.surls = surls;
 
-	gfal_srm_external_call.srm_context_init(&context, endpoint, errbuf, err_size, gfal_get_verbose());	
+    srm_context_t context = gfal_srm_ifce_context_setup(opts->handle, endpoint, errbuf, GFAL_URL_MAX_LEN, &tmp_err);
 
-	gfal_log(GFAL_VERBOSE_TRACE, "    [gfal_srm_putdone_srmv2_internal] start srm put done on %s", surls[0]);	
-	ret = gfal_srm_external_call.srm_put_done(&context,&putdone_input, &statuses);
-	if(ret < 0){
-		g_set_error(&tmp_err,0,errno,"call to srm_ifce error: %s",errbuf);
-	} else{
-    	 ret = gfal_srm_convert_filestatuses_to_GError(statuses, ret, &tmp_err);
-    	 gfal_srm_external_call.srm_srmv2_filestatus_delete(statuses, n_surl);
-	}
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-	return ret;	
+    if(context){
+        gfal_log(GFAL_VERBOSE_TRACE, "    [gfal_srm_putdone_srmv2_internal] start srm put done on %s", surls[0]);
+        ret = gfal_srm_external_call.srm_put_done(context,&putdone_input, &statuses);
+        if(ret < 0){
+            g_set_error(&tmp_err,0,errno,"call to srm_ifce error: %s",errbuf);
+        } else{
+             ret = gfal_srm_convert_filestatuses_to_GError(statuses, ret, &tmp_err);
+             gfal_srm_external_call.srm_srmv2_filestatus_delete(statuses, n_surl);
+        }
+    }
+    gfal_srm_ifce_context_release(context);
+    G_RETURN_ERR(ret, tmp_err, err);
 }
 
 int gfal_srm_putdone(gfal_srmv2_opt* opts , char** surls, char* token,  GError** err){
@@ -434,9 +430,7 @@ int gfal_srm_putdone(gfal_srmv2_opt* opts , char** surls, char* token,  GError**
 	}
 	gfal_log(GFAL_VERBOSE_TRACE, "   [gfal_srm_putdone] <-");
 
-	if(tmp_err)
-		g_propagate_prefixed_error(err, tmp_err, "[%s]", __func__);
-	return ret;		
+    G_RETURN_ERR(ret, tmp_err, err);
 	
 }
 
@@ -448,17 +442,19 @@ int gfal_srm_putdone_simple(plugin_handle * handle , const char* surl, char* tok
 
 int srmv2_abort_request_internal(gfal_srmv2_opt* opts , char* endpoint, char* req_token,  GError** err){
     GError* tmp_err=NULL;
-    struct srm_context context;
-    int ret=0;
+    int ret=-1;
 
     char errbuf[GFAL_URL_MAX_LEN] = {0};
 
-    gfal_srm_ifce_context_init(&context, opts->handle, endpoint,
-                                  errbuf, GFAL_URL_MAX_LEN, &tmp_err);
+    srm_context_t context = gfal_srm_ifce_context_setup(opts->handle, endpoint, errbuf, GFAL_URL_MAX_LEN, &tmp_err);
 
-    if((ret = srm_abort_request(&context, req_token)) < 0){
-        g_set_error(&tmp_err,0,errno,"SRMv2 abort request error : %s",errbuf);
+    if(context){
+        if((ret = srm_abort_request(context, req_token)) < 0){
+            g_set_error(&tmp_err,0,errno,"SRMv2 abort request error : %s",errbuf);
+        }
     }
+    gfal_srm_ifce_context_release(context);
+
     G_RETURN_ERR(ret, tmp_err, err);
 }
 
