@@ -23,12 +23,43 @@
  * @author Devresse Adrien
  * @version 2.0.0
  */
- 
+
+// gfal2 uses 64 bits offset size  by default
+#ifndef _FILE_OFFSET_BITS
+#  define _FILE_OFFSET_BITS 64
+#endif
+
+//
+// global context operations
+//
 #include <global/gfal_global.h>
+
+//
+// parameter and configuration api
+//
 #include <config/gfal_config.h>
+
+//
+// log  api
+//
 #include <logger/gfal_logger.h>
-#include <posix/gfal_posix_api.h> 
+
+//
+// main gfal 2.0 API for file operations
+//
 #include <file/gfal_file_api.h>
+
+
+//
+// operation control api
+//
+#include <cancel/gfal_cancel.h>
+
+//
+// gfal 1.0 compatibility layer
+//
+#include <posix/gfal_posix_api.h> 
+
  
 
 #endif  //_GFAL2_API_
