@@ -45,12 +45,7 @@ int gfal_posix_internal_rename(const char* oldpath, const char* newpath){
 		return -1;
 	}
 
-	if( oldpath == NULL || newpath == NULL){
-		g_set_error(&tmp_err, 0, EFAULT, " oldpath/newpath is an incorrect argument");
-	}else{	
-        ret = gfal_plugin_renameG(handle, oldpath, newpath, &tmp_err);
-	}
-	
+    ret = gfal2_rename(handle, oldpath, newpath, &tmp_err);
 	if(tmp_err){
 		gfal_posix_register_internal_error(handle, "[gfal_rename]", tmp_err);
 		errno = tmp_err->code;
